/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import cz.xelfi.quoridor.Move;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
//import javax.xml.bind.annotation.XmlAccessType;
//import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;

/**
 *
 * @author Martin Rexa
 */

@XmlRootElement
//@XmlAccessorType(XmlAccessType.FIELD)
public class OpeningNodeView {
    @XmlAttribute
    String code;
    @XmlElementWrapper(name="whiteGames")
    @XmlElement(name="gameId")
    List<GameId> whiteGames;
    @XmlElementWrapper(name="blackGames")
    @XmlElement(name="gameId")
    List<GameId> blackGames;
    Map<Move, OpeningNodeViewEntry> entries;
    boolean empty;

    public OpeningNodeView(){
    }

    public OpeningNodeView(String code){
        this.code = code;
        entries = new HashMap<Move, OpeningNodeViewEntry>();
        whiteGames = new ArrayList<GameId>();
        blackGames = new ArrayList<GameId>();
        empty = true;
    }

    @XmlElement
    public String getNodeCode(){
        return code;
    }

    public void processGame(Move m, String code, GameId gId){
        OpeningNodeViewEntry e = entries.get(m);
        if(e == null){
            e = new OpeningNodeViewEntry(m,code);
            entries.put(m, e);
        }
        e.processGame(gId);
        empty = false;
    }

    @XmlElementWrapper(name="children")
    @XmlElement(name="item")
    public Set<OpeningNodeViewEntry> getChildren(){
        Set<OpeningNodeViewEntry> result = new HashSet<OpeningNodeViewEntry>();
        for(Map.Entry<Move, OpeningNodeViewEntry> e: entries.entrySet()){
            result.add(e.getValue());
        }
        return result;
    }


    public void addFinishedGame(GameId gId){
        if(gId.getStatus().equals(GameStatus.whiteWon))
            whiteGames.add(gId);
        if(gId.getStatus().equals(GameStatus.blackWon))
            blackGames.add(gId);
        empty = false;
    }

    @XmlAttribute
    public int getWhiteCount(){
        return whiteGames.size();
    }

    @XmlAttribute
    public int getBlackCount(){
        return blackGames.size();
    }


    @XmlAttribute
    public boolean getChildrenNotEmpty(){
        return !entries.isEmpty();
    }

    public boolean isEmpty(){
        return this.empty;
    }

}
