/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.Fence.Orientation;
import org.junit.Test;
import static org.junit.Assert.*;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.webidor.Game;
import java.util.Map;

/**
 *
 * @author Martin Rexa
 */
public class OpeningsTest extends Object {
    
    public OpeningsTest() throws Exception {
    }

    @Test public void testMirrorMoves() throws Exception {
        assertEquals("bad symetric move", Move.NORTH.getMirrorMove(), Move.NORTH);
        assertEquals("bad symetric move", Move.SOUTH.getMirrorMove(), Move.SOUTH);
        assertEquals("bad symetric move", Move.WEST.getMirrorMove(), Move.EAST);
        assertEquals("bad symetric move", Move.EAST.getMirrorMove(), Move.WEST);
        assertEquals("bad symetric move", Move.fence('A', 1, Orientation.HORIZONTAL).getMirrorMove(), Move.fence('H', 1, Orientation.HORIZONTAL));
        assertEquals("bad symetric move", Move.fence('D', 3, Orientation.VERTICAL).getMirrorMove(), Move.fence('E', 3, Orientation.VERTICAL));
    }

    @Test public void testMirrorJump() throws Exception {
        Game g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.EAST, new java.util.Date());
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.valueOf("WW"), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        OpeningTree t = new OpeningTree();
        t.processGame(g);
        OpeningTreeNode n = t.root;
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.valueOf("EE"));
        assertEquals("bad number of children", n.children.size(), 0);
        for(Map.Entry e: n.children.entrySet()){
            System.out.println(e.getKey());
        }
    }

    @Test public void testOpeningTreeNodes() throws Exception {
        OpeningTree t = new OpeningTree();
        assertEquals("bad number of nodes", t.nodes.size(), 1);
        Game g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.EAST, new java.util.Date());
        g.apply("b", Move.EAST, new java.util.Date());
        g.apply("w", Move.WEST, new java.util.Date());
        g.apply("b", Move.WEST, new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 9);
        g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.fence('A', 1, Orientation.HORIZONTAL), new java.util.Date());
        g.apply("b", Move.fence('C', 1, Orientation.VERTICAL), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 13);
        g = new Game("w","b");
        g.apply("w", Move.NORTH, new java.util.Date());
        g.apply("b", Move.SOUTH, new java.util.Date());
        g.apply("w", Move.fence('C', 1, Orientation.VERTICAL), new java.util.Date());
        g.apply("b", Move.fence('A', 1, Orientation.HORIZONTAL), new java.util.Date());
        g.apply("w", Move.RESIGN, new java.util.Date());
        t.processGame(g);
        assertEquals("bad number of nodes", t.nodes.size(), 15);
        OpeningTreeNode n = t.root;
        n = n.children.get(Move.NORTH);
        n = n.children.get(Move.SOUTH);
        OpeningTreeNode n1 = n;
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.WEST);
        n = n.children.get(Move.EAST);
        n = n.children.get(Move.EAST);
        assertEquals("different nodes", n.hashCode, n1.hashCode);
        n = n.children.get(Move.fence('F', 1, Orientation.VERTICAL));
        n = n.children.get(Move.fence('H', 1, Orientation.HORIZONTAL));
        assertEquals("bad number of children", n.children.size(), 0);
    }

}
